@@ -12,7 +12,7 @@ use feature ();
 package MooX::Pression;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.013';
+our $VERSION   = '0.014';
 
 use Keyword::Declare;
 use B::Hooks::EndOfScope;
@@ -555,6 +555,75 @@ sub _handle_modifier_keyword {
 	}
 }
 
+sub _handle_package_keyword {
+	my ($me, $kind, $name, $code, $has_sig, $sig, $plus, $opts) = @_;
+	
+	if ($kind eq 'abstract') {
+		$kind = 'class';
+		$code = "{ q[$me]->_abstract(1);  $code }";
+	}
+	
+	if ($kind eq 'interface') {
+		$kind = 'role';
+		$code = "{ q[$me]->_interface(1); $code }";
+	}
+	
+	if ($name and $has_sig) {
+		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $code);
+		sprintf(
+			'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => { code => %s, named => %d, signature => %s }; use MooX::Pression::_Gather -unparent;',
+			B::perlstring("$plus$name"),
+			B::perlstring("$kind\_generator:$plus$name"),
+			$munged_code,
+			!!$signature_is_named,
+			$type_params_stuff,
+		);
+	}
+	elsif ($has_sig) {
+		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
+		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $code);
+		sprintf(
+			'q[%s]->anonymous_generator(%s => { code => %s, named => %d, signature => %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$kind,
+			$munged_code,
+			!!$signature_is_named,
+			$type_params_stuff,
+			B::perlstring($opts->{toolkit}||'Moo'),
+			B::perlstring($opts->{prefix}),
+			B::perlstring($opts->{factory_package}),
+			B::perlstring($opts->{type_library}),
+		);
+	}
+	elsif ($name) {
+		$code
+			? sprintf(
+				'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => q[%s]->_package_callback(sub %s); use MooX::Pression::_Gather -unparent;',
+				B::perlstring("$plus$name"),
+				B::perlstring("$kind:$plus$name"),
+				$me,
+				$code,
+			)
+			: sprintf(
+				'use MooX::Pression::_Gather -gather, %s => {};',
+				B::perlstring("$kind:$plus$name"),
+			);
+	}
+	else {
+		$code ||= '{}';
+		sprintf(
+			'q[%s]->anonymous_package(%s => sub { do %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
+			$me,
+			$kind,
+			$code,
+			B::perlstring($opts->{toolkit}||'Moo'),
+			B::perlstring($opts->{prefix}),
+			B::perlstring($opts->{factory_package}),
+			B::perlstring($opts->{type_library}),
+		);
+	}
+}
 
 #
 # KEYWORDS/UTILITIES
@@ -604,118 +673,76 @@ sub import {
 	
 	# `class` keyword
 	#
-	keyword class ('+'? $plus, QualifiedIdentifier $classname, '(' $has_sig, SignatureList? $sig, ')', Block $classdfn) {
-		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
-		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
-		sprintf(
-			'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => { code => %s, named => %d, signature => %s }; use MooX::Pression::_Gather -unparent;',
-			B::perlstring("$plus$classname"),
-			B::perlstring("class_generator:$plus$classname"),
-			$munged_code,
-			!!$signature_is_named,
-			$type_params_stuff,
-		);
+	keyword class ('+'? $plus, QualifiedIdentifier $name, '(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(class => $name, $block, $has_sig, $sig,  $plus, \%opts);
 	}
-	keyword class ('+'? $plus, QualifiedIdentifier $classname, Block $classdfn) {
-		sprintf(
-			'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => q[%s]->_package_callback(sub %s); use MooX::Pression::_Gather -unparent;',
-			B::perlstring("$plus$classname"),
-			B::perlstring("class:$plus$classname"),
-			$me,
-			$classdfn,
-		);
+	keyword class ('+'? $plus, QualifiedIdentifier $name, Block $block) {
+		my $return = $me->_handle_package_keyword(class => $name, $block, 0,        undef, $plus, \%opts);
 	}
-	keyword class ('+'? $plus, QualifiedIdentifier $classname) {
-		sprintf(
-			'use MooX::Pression::_Gather -gather, %s => {};',
-			B::perlstring("class:$plus$classname"),
-		);
+	keyword class ('+'? $plus, QualifiedIdentifier $name) {
+		my $return = $me->_handle_package_keyword(class => $name, '',     0,        undef, $plus, \%opts);
 	}
-	keyword class ('(' $has_sig, SignatureList? $sig, ')', Block $classdfn) {
-		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
-		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
-		sprintf(
-			'q[%s]->anonymous_generator(class => { code => %s, named => %d, signature => %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
-			$me,
-			$munged_code,
-			!!$signature_is_named,
-			$type_params_stuff,
-			B::perlstring($opts{toolkit}||'Moo'),
-			B::perlstring($opts{prefix}),
-			B::perlstring($opts{factory_package}),
-			B::perlstring($opts{type_library}),
-		);
+	keyword class ('(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(class => undef, $block, $has_sig, $sig,  '',    \%opts);
 	}
-	keyword class (Block? $classdfn) {
-		$classdfn ||= '{}';
-		sprintf(
-			'q[%s]->anonymous_package(class => sub { do %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
-			$me,
-			$classdfn,
-			B::perlstring($opts{toolkit}||'Moo'),
-			B::perlstring($opts{prefix}),
-			B::perlstring($opts{factory_package}),
-			B::perlstring($opts{type_library}),
-		);
+	keyword class (Block? $block) {
+		my $return = $me->_handle_package_keyword(class => undef, $block, 0,        undef, '',    \%opts);
 	}
-	
+
+	# `abstract` keyword
+	#
+	keyword abstract ('class', '+'? $plus, QualifiedIdentifier $name, '(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(abstract => $name, $block, $has_sig, $sig,  $plus, \%opts);
+	}
+	keyword abstract ('class', '+'? $plus, QualifiedIdentifier $name, Block $block) {
+		my $return = $me->_handle_package_keyword(abstract => $name, $block, 0,        undef, $plus, \%opts);
+	}
+	keyword abstract ('class', '+'? $plus, QualifiedIdentifier $name) {
+		my $return = $me->_handle_package_keyword(abstract => $name, '',     0,        undef, $plus, \%opts);
+	}
+	keyword abstract ('class', '(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(abstract => undef, $block, $has_sig, $sig,  '',    \%opts);
+	}
+	keyword abstract ('class', Block? $block) {
+		my $return = $me->_handle_package_keyword(abstract => undef, $block, 0,        undef, '',    \%opts);
+	}
+
 	# `role` keyword
 	#
-	keyword role (QualifiedIdentifier $classname, '(' $has_sig, SignatureList? $sig, ')', Block $classdfn) {
-		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
-		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
-		sprintf(
-			'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => { code => %s, named => %d, signature => %s }; use MooX::Pression::_Gather -unparent;',
-			B::perlstring($classname),
-			B::perlstring('role_generator:'.$classname),
-			$munged_code,
-			!!$signature_is_named,
-			$type_params_stuff,
-		);
+	keyword role (QualifiedIdentifier $name, '(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(role => $name, $block, $has_sig, $sig,  '',    \%opts);
 	}
-	keyword role (QualifiedIdentifier $classname, Block $classdfn) {
-		sprintf(
-			'use MooX::Pression::_Gather -parent => %s; use MooX::Pression::_Gather -gather, %s => q[%s]->_package_callback(sub %s); use MooX::Pression::_Gather -unparent;',
-			B::perlstring($classname),
-			B::perlstring('role:'.$classname),
-			$me,
-			$classdfn,
-		);
+	keyword role (QualifiedIdentifier $name, Block $block) {
+		my $return = $me->_handle_package_keyword(role => $name, $block, 0,        undef, '',    \%opts);
 	}
-	keyword role (QualifiedIdentifier $classname) {
-		sprintf(
-			'use MooX::Pression::_Gather -gather, %s => {};',
-			B::perlstring('role:'.$classname),
-		);
+	keyword role (QualifiedIdentifier $name) {
+		my $return = $me->_handle_package_keyword(role => $name, '',     0,        undef, '',    \%opts);
 	}
-	keyword role ('(' $has_sig, SignatureList? $sig, ')', Block $classdfn) {
-		my ($signature_is_named, $signature_var_list, $type_params_stuff, $extra) = $handle_signature_list->($sig);
-		my $munged_code = sprintf('sub { q(%s)->_package_callback(sub { my ($generator,%s)=(shift,@_); %s; do %s }, @_) }', $me, $signature_var_list, $extra, $classdfn);
-		sprintf(
-			'q[%s]->anonymous_generator(role => { code => %s, named => %d, signature => %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
-			$me,
-			$munged_code,
-			!!$signature_is_named,
-			$type_params_stuff,
-			B::perlstring($opts{toolkit}||'Moo'),
-			B::perlstring($opts{prefix}),
-			B::perlstring($opts{factory_package}),
-			B::perlstring($opts{type_library}),
-		);
+	keyword role ('(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(role => undef, $block, $has_sig, $sig,  '',    \%opts);
 	}
-	keyword role (Block? $classdfn) {
-		$classdfn ||= '{}';
-		sprintf(
-			'q[%s]->anonymous_package(role => sub { do %s }, toolkit => %s, prefix => %s, factory_package => %s, type_library => %s)',
-			$me,
-			$classdfn,
-			B::perlstring($opts{toolkit}||'Moo'),
-			B::perlstring($opts{prefix}),
-			B::perlstring($opts{factory_package}),
-			B::perlstring($opts{type_library}),
-		);
+	keyword role (Block? $block) {
+		my $return = $me->_handle_package_keyword(role => undef, $block, 0,        undef, '',    \%opts);
 	}
-	
+
+	# `interface` keyword
+	#
+	keyword interface (QualifiedIdentifier $name, '(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(interface => $name, $block, $has_sig, $sig,  '',    \%opts);
+	}
+	keyword interface (QualifiedIdentifier $name, Block $block) {
+		my $return = $me->_handle_package_keyword(interface => $name, $block, 0,        undef, '',    \%opts);
+	}
+	keyword interface (QualifiedIdentifier $name) {
+		my $return = $me->_handle_package_keyword(interface => $name, '',     0,        undef, '',    \%opts);
+	}
+	keyword interface ('(' $has_sig, SignatureList? $sig, ')', Block $block) {
+		my $return = $me->_handle_package_keyword(interface => undef, $block, $has_sig, $sig,  '',    \%opts);
+	}
+	keyword interface (Block? $block) {
+		my $return = $me->_handle_package_keyword(interface => undef, $block, 0,        undef, '',    \%opts);
+	}
+
 	# `toolkit` keyword
 	#
 	keyword toolkit (Identifier $tk, '(', QualifiedIdentifier|Comma @imports, ')') {
@@ -957,6 +984,14 @@ sub _end {
 	shift;
 	$OPTS{end} = shift;
 }
+sub _interface {
+	shift;
+	$OPTS{interface} = shift;
+}
+sub _abstract {
+	shift;
+	$OPTS{abstract} = shift;
+}
 sub _with {
 	shift;
 	push @{ $OPTS{with}||=[] }, @_;
@@ -1003,7 +1038,7 @@ sub _modifier {
 #{
 #	package MooX::Pression::Anonymous::Package;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.013';
+#	our $VERSION   = '0.014';
 #	use overload q[""] => sub { ${$_[0]} }, fallback => 1;
 #	sub DESTROY {}
 #	sub AUTOLOAD {
@@ -1014,7 +1049,7 @@ sub _modifier {
 #	
 #	package MooX::Pression::Anonymous::Class;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.013';
+#	our $VERSION   = '0.014';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub new {
 #		my $me = shift;
@@ -1027,12 +1062,12 @@ sub _modifier {
 #	
 #	package MooX::Pression::Anonymous::Role;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.013';
+#	our $VERSION   = '0.014';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	
 #	package MooX::Pression::Anonymous::ParameterizableClass;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.013';
+#	our $VERSION   = '0.014';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -1046,7 +1081,7 @@ sub _modifier {
 #
 #	package MooX::Pression::Anonymous::ParameterizableRole;
 #	our $AUTHORITY = 'cpan:TOBYINK';
-#	our $VERSION   = '0.013';
+#	our $VERSION   = '0.014';
 #	our @ISA       = qw(MooX::Pression::Anonymous::Package);
 #	sub generate_package {
 #		my $me  = shift;
@@ -1359,6 +1394,21 @@ It is possible to prefix a class name with a plus sign:
 Now the employee class will be named C<MyApp::Person::Employee> instead of
 the usual C<MyApp::Employee>.
 
+Classes can be declared as abstract:
+
+  package MyApp {
+    use MooX::Pression;
+    abstract class Animal {
+      class Cat;
+      class Dog;
+    }
+  }
+
+For abstract classes, there is no constructor or factory, so you cannot create
+an Animal instance directly; but you can create instances of the subclasses.
+It is usually better to use roles than abstract classes, but sometimes the
+abstract class makes more intuitive sense.
+
 =head3 C<< role >>
 
 Define a very basic role:
@@ -1376,6 +1426,32 @@ This is just the same as C<class> but defines a role instead of a class.
 Roles cannot be nested within each other, nor can roles be nested in classes,
 nor classes in roles.
 
+=head3 C<< interface >>
+
+An interface is a lightweight role. It cannot define attributes, methods,
+multimethods, or method modifiers, but otherwise functions as a role.
+(It may have C<requires> statements and define constants.)
+
+  package MyApp;
+  use MooX::Pression;
+  
+  interface Serializer {
+    requires serialize;
+  }
+  
+  interface Deserializer {
+    requires deserialize;
+  }
+  
+  class MyJSON {
+    with Serializer, Deserialize;
+    method serialize   ($value) { ... } 
+    method deserialize ($value) { ... } 
+  }
+  
+  my $obj = MyApp->new_myjson;
+  $obj->does('MyApp::Serializer');   # true
+
 =head3 C<< toolkit >>
 
 Use a different toolkit instead of Moo.
@@ -2744,6 +2820,34 @@ They work nicely in MooX::Pression though.
 Please report any bugs to
 L<http://rt.cpan.org/Dist/Display.html?Queue=MooX-Pression>.
 
+=head1 TODO
+
+=head2 Better type constraint parsing
+
+Better parsing for type constraints would be good. Right now, type
+constraints are parsed from the signature by looking for a non-numeric
+word character and grabbing everything up to the first whitespace character.
+
+A good grammar for type constraints would be something like: types can
+Perl (possibly qualified) identifiers (like "Foo" or "Foo::Bar"). They
+can be optionally prefixed with "~", optionally suffixed with "[...]",
+combined with "|" and "&" infix operators, and may use parentheses to
+express precedence. The grammar for the contents of the square brackets
+is that it's a list separated by commas and/or fat commas. List items
+may be strings, numbers, quoted regexps, or other type constraints.
+(Those other type constraints can themselves have things like infix
+operators, "[...]", etc.)
+
+The regexp doesn't really need to break down all the components; just
+figure out where it ends, so the type can be passed to L<Type::Parser>.
+
+=head2 Plugin system
+
+MooX::Pression can often load MooX/MouseX/MooseX plugins and work
+fine with them, but some things won't work, like plugins that rely on
+being able to wrap C<has>. So it would be nice to have a plugin system
+that extensions can hook into.
+
 =head1 SEE ALSO
 
 Less magic version:
